let fs = require('mz/fs');
let http = require('http');
let chalk = require('chalk');
let ejs = require('ejs');
let path = require('path');
let url = require('url');
let mime = require('mime');
let zlib = require('zlib');

let template = fs.readFileSync(path.join(__dirname, 'template.html'), 'utf-8');
class Server{
    constructor(config){
        this.dir = config.dir;
        this.host = config.host;
        this.port = config.port;
        this.template = template;
    }
    async handleServer(req, res){
        let {pathname} = url.parse(req.url, true);
        let absPath = path.join(this.dir, decodeURIComponent(pathname));
        try{
            let statObj = await fs.stat(absPath);
            if (statObj.isDirectory()) {
                let dirs = await fs.readdir(absPath);
                let content = ejs.render(this.template, {arr: dirs.map((item) => {
                    let fPath = path.join(pathname, item)
                        let fState = fs.statSync(path.join(this.dir, fPath));
                        let isDir = fState.isDirectory();
                    return {
                        name: item,
                        path: fPath,
                        isDir
                    }
                })});
                res.setHeader('Content-Type', mime.getType(absPath) + ';charset=urf8');
                res.end(content);
            } else {
                this.sendFile(req, res, absPath);
            }
        }catch(e) {
            console.log('err', e);
            this.sendErr(res);
        }
    }
    sendErr(res){
        res.statusCode = 404;
        res.end('NOT FOUND');
    }
    sendFile(req, res, absPath){
        res.statusCode = 200;
        res.setHeader('Content-Type', mime.getType(absPath) + ';charset=urf8');
        let encoding = req.headers['accept-encoding'];
        let range = req.headers['range'];
        let start, end;
        let total = fs.statSync(absPath).size;
        if (range) {
            [,start,end] = range.match(/(\d*)-(\d*)/);
            start = start || 0;
            end = end || total;
            res.setHeader('Content-Range',`bytes ${start}-${end}/${total}`);
            start = Number(start);
            end = Number(end);
            res.statusCode = 206;
        }
        if (/\bgzip\b/.test(encoding)){
            res.setHeader('Content-Encoding', 'gzip');
            fs.createReadStream(absPath,{start,end}).pipe(zlib.createGzip()).pipe(res);
            return;
        }
        if (/\bdeflate\b/.test(encoding)){
            res.setHeader('Content-Encoding', 'deflate');
            fs.createReadStream(absPath,{start,end}).pipe(zlib.createGzip()).pipe(res);
            return;
        }
        fs.createReadStream(absPath,{start,end}).pipe(res);
    }
    start(){
        let server = http.createServer(this.handleServer.bind(this));
        server.listen(this.port, this.host, () => {
            console.log(chalk.yellow(`Starting up http-server, serving ${this.dir} \r\nAvailable on\r\n`));
            console.log(chalk.green(`http://${this.host}:${this.port}`));
        })
    }
}
module.exports = Server;